/* $Id: init.c,v 1.5 2006/01/21 16:17:14 cegger Exp $
***************************************************************************

   X server for LibGGI - GC (graphics context) acceleration

   Copyright (C) 1997      Michael Krause	[rawstyle@ms.demo.org]
   Copyright (C) 1998-2000 Marcus Sundberg	[marcus@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

***************************************************************************
*/


#define PSZ 8
#include <ggi/gg.h>

#include "scrnintstr.h"
#include "gcstruct.h"

#include "xggi.h"
#include "xggi_misc.h"
#include "xggi_struct.h"

#include "gc/gc.h"
#include "screen/screen.h"


Bool xggiAccelInitBackingStore(int scr)
{
	ScreenPtr pScreen = SCREEN_SCR(scr);

	pScreen->BackingStoreFuncs.SaveAreas = xggiSaveRestoreAreas;
	pScreen->BackingStoreFuncs.RestoreAreas = xggiSaveRestoreAreas;

	return TRUE;
}	/* xggiAccelInitBackingStore */


/*
 * xggiAccelInit
 *
 */
Bool xggiAccelInit(int scr)
{
	ScreenPtr pScreen;
	xggiScreenBackupPtr pBackup;


#ifdef XGGI_DEBUG
	ErrorF("scr: %d, depth: %d, bpp:%d\n",
		scr, MASTER_VISUAL_DEPTH(scr), MASTER_VISUAL_BPP(scr));
#endif

	pScreen = SCREEN_SCR(scr);
	pBackup = SCREEN_SCRBACKUP(scr);

	pScreen->CopyWindow = xggiCopyWindow;
	pScreen->GetImage = xggiGetImage;
	pScreen->CreateGC = xggiCreateGC;
	pScreen->CloseScreen = xggiCloseScreen;

	pScreen->PaintWindowBackground = xggiPaintWindow;
	pScreen->PaintWindowBorder = xggiPaintWindow;

	xggiAccelInitBackingStore(scr);


	switch (SCREEN_ROOTDEPTH(scr)) {
	case 1:
		pBackup->GetImage = mfbGetImage;
		pBackup->CreateGC = mfbCreateGC;
		pBackup->PaintWindowBackground = mfbPaintWindow;
		pBackup->PaintWindowBorder = mfbPaintWindow;

		pScreen->GetSpans = mfbGetSpans;
		if (!mfbAllocatePrivates(pScreen, NULL, NULL)) {
			ErrorF("mfbAllocatePrivates failed\n");
			return FALSE;
		}	/* if */
		pScreen->CreateWindow = mfbCreateWindow;
		pScreen->DestroyWindow = mfbDestroyWindow;
		pScreen->PositionWindow = mfbPositionWindow;
		pScreen->ChangeWindowAttributes = mfbChangeWindowAttributes;
		pScreen->RealizeWindow = mfbMapWindow;
		pScreen->UnrealizeWindow = mfbUnmapWindow;
		pScreen->CreatePixmap = mfbCreatePixmap;
		pScreen->DestroyPixmap = mfbDestroyPixmap;
		mfbRegisterCopyPlaneProc(pScreen, mfbCopyPlane);

		pBackup->CloseScreen = mfbCloseScreen;
		break;
	case 8:
		pBackup->GetImage = cfbGetImage;
		pBackup->CreateGC = cfbCreateGC;
		pBackup->PaintWindowBackground = cfbPaintWindow;
		pBackup->PaintWindowBorder = cfbPaintWindow;

		pScreen->GetSpans = cfbGetSpans;
		if (!cfbAllocatePrivates(pScreen, NULL, NULL)) {
			ErrorF("cfbAllocatePrivates failed\n");
			return FALSE;
		}	/* if */
		pScreen->CreateWindow = cfbCreateWindow;
		pScreen->DestroyWindow = cfbDestroyWindow;
		pScreen->PositionWindow = cfbPositionWindow;
		pScreen->ChangeWindowAttributes = cfbChangeWindowAttributes;
		pScreen->RealizeWindow = cfbMapWindow;
		pScreen->UnrealizeWindow = cfbUnmapWindow;
		pScreen->CreatePixmap = cfbCreatePixmap;
		pScreen->DestroyPixmap = cfbDestroyPixmap;
		mfbRegisterCopyPlaneProc(pScreen, cfbCopyPlane);

		pBackup->CloseScreen = cfbCloseScreen;
		break;
	case 15:
	case 16:
		pBackup->GetImage = cfb16GetImage;
		pBackup->CreateGC = cfb16CreateGC;
		pBackup->PaintWindowBackground = cfb16PaintWindow;
		pBackup->PaintWindowBorder = cfb16PaintWindow;

		pScreen->GetSpans = cfb16GetSpans;
		if (!cfb16AllocatePrivates(pScreen, NULL, NULL)) {
			ErrorF("cfb16AllocatePrivates failed\n");
			return FALSE;
		}	/* if */
		pScreen->CreateWindow = cfb16CreateWindow;
		pScreen->DestroyWindow = cfb16DestroyWindow;
		pScreen->PositionWindow = cfb16PositionWindow;
		pScreen->ChangeWindowAttributes = cfb16ChangeWindowAttributes;
		pScreen->RealizeWindow = cfb16MapWindow;
		pScreen->UnrealizeWindow = cfb16UnmapWindow;
		pScreen->CreatePixmap = cfb16CreatePixmap;
		pScreen->DestroyPixmap = cfb16DestroyPixmap;
		mfbRegisterCopyPlaneProc(pScreen, cfb16CopyPlane);

		pBackup->CloseScreen = cfb16CloseScreen;
#if 0
		pScreen->CreateScreenResources = cfb16CreateScreenResources;
#endif   
		break;
	case 24:
	case 32:
		if (MASTER_VISUAL(scr).bitsPerPixel == 24) {
			pBackup->GetImage = cfb24GetImage;
			pBackup->CreateGC = cfb24CreateGC;
			pBackup->PaintWindowBackground = cfb24PaintWindow;
			pBackup->PaintWindowBorder = cfb24PaintWindow;

			pScreen->GetSpans = cfb24GetSpans;
			if (!cfb24AllocatePrivates(pScreen, NULL, NULL)) {
				ErrorF("cfb24AllocatePrivates failed\n");
				return FALSE;
			}	/* if */
			pScreen->CreateWindow = cfb24CreateWindow;
			pScreen->DestroyWindow = cfb24DestroyWindow;
			pScreen->PositionWindow = cfb24PositionWindow;
			pScreen->ChangeWindowAttributes
				= cfb24ChangeWindowAttributes;
			pScreen->RealizeWindow = cfb24MapWindow;
			pScreen->UnrealizeWindow = cfb24UnmapWindow;
			pScreen->CreatePixmap = cfb24CreatePixmap;
			pScreen->DestroyPixmap = cfb24DestroyPixmap;
			mfbRegisterCopyPlaneProc(pScreen, cfb24CopyPlane);

			pBackup->CloseScreen = cfb24CloseScreen;
#if 0
			pScreen->CreateScreenResources
				= cfb24CreateScreenResources;
#endif
		} else {
			pBackup->GetImage = cfb32GetImage;
			pBackup->CreateGC = cfb32CreateGC;
			pBackup->PaintWindowBackground = cfb32PaintWindow;
			pBackup->PaintWindowBorder = cfb32PaintWindow;

			pScreen->GetSpans = cfb32GetSpans;
			if (!cfb32AllocatePrivates(pScreen, NULL, NULL)) {
				ErrorF("cfb32AllocatePrivates failed\n");
				return FALSE;
			}	/* if */
			pScreen->CreateWindow = cfb32CreateWindow;
			pScreen->DestroyWindow = cfb32DestroyWindow;
			pScreen->PositionWindow = cfb32PositionWindow;
			pScreen->ChangeWindowAttributes
				= cfb32ChangeWindowAttributes;
			pScreen->RealizeWindow = cfb32MapWindow;
			pScreen->UnrealizeWindow = cfb32UnmapWindow;
			pScreen->CreatePixmap = cfb32CreatePixmap;
			pScreen->DestroyPixmap = cfb32DestroyPixmap;
			mfbRegisterCopyPlaneProc(pScreen, cfb32CopyPlane);

			pBackup->CloseScreen = cfb32CloseScreen;
#if 0
			pScreen->CreateScreenResources
				= cfb32CreateScreenResources;
#endif
		}	/* if */
		break;

	default:
		FatalError("root depth %d not supported\n",
			SCREEN_ROOTDEPTH(scr));
	}	/* switch */


	pScreen->CreateColormap = cfbInitializeColormap;
	pScreen->DestroyColormap = (DestroyColormapProcPtr)NoopDDA;
	if (SCREEN_ROOTDEPTH(scr) == 1) {
		pScreen->ResolveColor = mfbResolveColor;
	} else {
		pScreen->ResolveColor = cfbResolveColor;
	}	/* if */
	pScreen->BitmapToRegion = mfbPixmapToRegion;

	return TRUE;
}	/* xggiAccelInit */
